#ifndef DOMAIN_PIN_H
#define DOMAIN_PIN_H

#include "display.h"
#include "value.h"

#include <string>



namespace domain {

class PinConnection
{
public:
    enum ConnectionType {
        CHECK_ONLY,
        AUTOMATICALLY
    };

private:
    std::string message;
    ConnectionType type;

public:
    PinConnection(std::string message, ConnectionType type);

    ConnectionType getType();
    std::string getMessage();
};

class Pin
{
public:
    enum PinType {
        IN_OUT,
        IN,
        OUT
    };

private:
    std::string name;
    PinType type;
    std::string tooltip;
    PinConnection connection;
    Display display;

    std::optional<std::vector<Value>> wires;

public:
    Pin(std::string name, PinType type, std::string tooltip, PinConnection connection, Display display, std::optional<std::vector<Value>> wires);

    std::string getName();
    PinType getType();
    std::string getTooltip();
    Display &getDisplay();
    PinConnection &getConnection();
    std::optional<std::vector<Value>> &getWires();
};

} // namespace domain

#endif // DOMAIN_PIN_H
