#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H

#include <QListWidget>
#include <QMainWindow>

#include <comdel/display/library_display.h>
#include <comdel/display/schema_display.h>

#include <comdel/domain/library.h>
#include <comdel/domain/schema.h>
#include <QPlainTextEdit>
#include <comdel/domain/comdelvalidator.h>

QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

    display::Library *libraryDisplay;
    display::Schema *schemaDisplay;

    std::optional<domain::Library> library = std::nullopt;
    domain::Schema* schema = nullptr;

    std::vector<domain::FunctionSignature> signatures;

    void setupUi();
    void clear();

private slots:
    void onLoadLibrary();
    void onLoadSchema();
    void onValidateSchema(bool toggled);

private:
    Ui::MainWindow *ui;
    QPlainTextEdit *log;
    std::vector<domain::ValidationError> validationErrors;
};
#endif // MAIN_WINDOW_H
